using System;
using System.Collections.Generic;
using System.Text;
using gehtsoft.indicore;

namespace ApplyStrategy
{
    class HistoryMgr // this class is for dealing with history price streams
    {
        private static string sLastQuoteID = string.Empty; // if last QuoteID was changed, we got new prices
        public static Dictionary<IntPtr, List<Ticket>> dictHistoryStreams = new Dictionary<IntPtr, List<Ticket>>(); // info about strategy instance and associated streams of historic prices
        public static void Attach2Instance(IntPtr stratInstance, IndicatorSource src, bool bCompleted, string sHistRequest, int iCookie, bool bTillNow)
        {
            // attach this price stream (IndicatorSource) to strategy instance (keep it in a dictionary of history streams, where key is a strategy instance, and value is a list of tickets with corresponding price streams)
            Ticket tkt = new Ticket(src, bCompleted, sHistRequest, iCookie, bTillNow);
            if (dictHistoryStreams.ContainsKey(stratInstance))
            {
                List<Ticket> lstTkt;
                dictHistoryStreams.TryGetValue(stratInstance, out lstTkt);
                if (lstTkt != null)
                {
                    lstTkt.Add(tkt);
                    dictHistoryStreams[stratInstance] = lstTkt;
                }
            }
            else
            {
                List<Ticket> listHistoryStreams = new List<Ticket>();
                listHistoryStreams.Add(tkt);
                dictHistoryStreams.Add(stratInstance, listHistoryStreams);
            }
        }
        public static void GetPriceHistory(IntPtr stratInstance, string sInstrument, string strBarSize, DateTime dtFrom, DateTime dtTo, int iNumCandles, bool bIsBid, bool bIsAsk, int iCookie, out IndicatorSource bidSrc, out IndicatorSource askSrc)
        {
            if ((stratInstance == IntPtr.Zero) && (strBarSize == "t1")) // ticks for strategy ("bid" and "ask")
            {
                PrepareTicks(sInstrument, out bidSrc, out askSrc);
            }
            else // steams from "host:getHistory"
            {
                GetPriceHistoryAsync(stratInstance, sInstrument, strBarSize, dtFrom, dtTo, iNumCandles, bIsBid, bIsAsk, iCookie, out bidSrc, out askSrc);
            }
        }
        public static void PrepareTicks(string sInstrument, out IndicatorSource bidSrc, out IndicatorSource askSrc) // Get ticks as source for strategy ("bid" and "ask")
        {
            MyIndicatorTickSource tickBidHist = new MyIndicatorTickSource(true, sInstrument);
            MyIndicatorTickSource tickAskHist = new MyIndicatorTickSource(false, sInstrument);
            // get historic ticks
            FXCore.MarketRateEnumAut rates = (FXCore.MarketRateEnumAut)Program.TradeDesk.GetPriceHistory(sInstrument, "t1", DateTime.FromOADate(0), DateTime.FromOADate(0), 300, true, true); // preload tick history
            foreach (FXCore.MarketRateAut rate in rates)
            {
                DateTime rateDt = rate.StartDate;
                tickBidHist.AddLast(rateDt, rate.BidClose);
                tickAskHist.AddLast(rateDt, rate.AskClose);
            }
            // get fresh ticks
            MyIndicatorTickSource tickBidBuffer = new MyIndicatorTickSource(true, sInstrument);
            MyIndicatorTickSource tickAskBuffer = new MyIndicatorTickSource(false, sInstrument);
            FXCore.TradeDeskEventEnumAut pending_events = (FXCore.TradeDeskEventEnumAut)Program.TradeDesk.GetPendingEvents();
            if (pending_events.Count > 0)
            {
                foreach (FXCore.TradeDeskEventAut pending_event in pending_events)
                {
                    if (pending_event.Kind == pending_event.KIND_CHANGE)
                    {
                        FXCore.TableAut aTable = (FXCore.TableAut)pending_event.Table;
                        if ((aTable != null) && (aTable.Type == "offers"))      // "tick" event
                        {
                            string rowId = pending_event.RowID;
                            FXCore.IRowAut row = (FXCore.IRowAut)aTable.FindRow("OfferID", rowId, 0);
                            if (row != null)
                            {
                                string sRowInstr = row.CellValue("Instrument").ToString();
                                if (sRowInstr == sInstrument)
                                {
                                    string strQuoteID = row.CellValue("QuoteID").ToString();
                                    if (strQuoteID != sLastQuoteID) // new prices
                                    {
                                        sLastQuoteID = strQuoteID;
                                        string strBid = row.CellValue("Bid").ToString();
                                        double dblBid = Double.Parse(strBid);
                                        string strAsk = row.CellValue("Ask").ToString();
                                        double dblAsk = Double.Parse(strAsk);
                                        string strDateTime = row.CellValue("Time").ToString();
                                        DateTime dt = Convert.ToDateTime(strDateTime);
                                        FXCore.TimeZoneConverterAut timeZoneCoverter = (FXCore.TimeZoneConverterAut)Program.TradeDesk.TimeZoneConverter;
                                        DateTime dtLocal = timeZoneCoverter.Convert(dt, timeZoneCoverter.ZONE_UTC, timeZoneCoverter.ZONE_LOCAL);
                                        tickBidBuffer.AddLast(dtLocal, dblBid);
                                        tickAskBuffer.AddLast(dtLocal, dblAsk);
                                    }
                                }
                            }
                        }
                    }
                }
            }

            MyIndicatorTickSource tickBid;
            MyIndicatorTickSource tickAsk;

            //Merge historic and fresh ticks
            MergeTicksHistWithTicksBuffer(sInstrument, tickBidHist, tickBidBuffer, tickAskHist, tickAskBuffer, out tickBid, out tickAsk);
            bidSrc = tickBid;
            askSrc = tickAsk;
        }
        public static void GetPriceHistoryAsync(IntPtr stratInstance, string sInstrument, string strBarSize, DateTime dtFrom, DateTime dtTo, int iNumCandles, bool bIsBid, bool bIsAsk, int iCookie, out IndicatorSource bidSrc, out IndicatorSource askSrc)
        {
            if (strBarSize == "t1")
            {
                bidSrc = new MyIndicatorTickSource(true, sInstrument);
                askSrc = new MyIndicatorTickSource(false, sInstrument);
            }
            else
            {
                bidSrc = new MyIndicatorBarSource(true, sInstrument);
                askSrc = new MyIndicatorBarSource(false, sInstrument);
            }
            string strReq = Program.TradeDesk.GetPriceHistoryAsync(sInstrument, strBarSize, dtFrom, dtTo, iNumCandles, bIsBid, bIsAsk);

            bool bTillNow = false;
            if (dtTo == DateTime.FromOADate(0))
                bTillNow = true;

            if (bIsBid)
                HistoryMgr.Attach2Instance(stratInstance, bidSrc, false, strReq, iCookie, bTillNow);
            if (bIsAsk)
                HistoryMgr.Attach2Instance(stratInstance, askSrc, false, strReq, iCookie, bTillNow);
        }
        private static bool IsSamePatternStarted(int iHistNumber, MyIndicatorTickSource tickBidHist, MyIndicatorTickSource tickBidBuffer, MyIndicatorTickSource tickAskHist, MyIndicatorTickSource tickAskBuffer)
        {
            // check whether there is an exact match between history ticks and tick buffer starting with the period iHistNumber
            // if the answer is "yes" it means overlapping of these streams
            try
            {
                int iHistPricesLeft = tickAskHist.size() - iHistNumber;
                for (int i = 0; i < iHistPricesLeft; i++) // check whether we have the all leftover historic ticks in the buffer
                {
                    DateTime dtHist = tickAskHist.getDate(i + iHistNumber);
                    double dblTickHistAsk = tickAskHist.getTick(i + iHistNumber);
                    double dblTickHistBid = tickBidHist.getTick(i + iHistNumber);
                    DateTime dtBuffer = tickAskBuffer.getDate(i);
                    double dblTickBufferAsk = tickAskBuffer.getTick(i);
                    double dblTickBufferBid = tickBidBuffer.getTick(i);
                    if ((dtHist != dtBuffer) || (dblTickHistAsk != dblTickBufferAsk) || (dblTickHistBid != dblTickBufferBid))
                    {
                        return false;
                    }
                }
                return true;
            }
            catch
            {
                return false;
            }
        }
        private static void MergeTicksHistWithTicksBuffer(string sInstrument, MyIndicatorTickSource tickBidHist, MyIndicatorTickSource tickBidBuffer, MyIndicatorTickSource tickAskHist, MyIndicatorTickSource tickAskBuffer, out MyIndicatorTickSource tickBid, out MyIndicatorTickSource tickAsk)
        {
            // merge "old" ticks from history and "new" ticks from tick buffer without overlapping
            tickBid = new MyIndicatorTickSource(true, sInstrument);
            tickAsk = new MyIndicatorTickSource(false, sInstrument);
            try
            {
                bool bAddWholeHist = false;
                DateTime dtBufferFirst;
                DateTime dtHistLast;
                if (tickAskBuffer.size() == 0)          // there are no ticks in the buffer => add the whole history
                {
                    bAddWholeHist = true;
                }
                else
                {
                    dtBufferFirst = tickAskBuffer.getDate(0);
                    int iHistSize = tickAskHist.size();
                    if (iHistSize > 0)
                    {
                        dtHistLast = tickAskHist.getDate(tickAskHist.size() - 1);
                        if (dtHistLast < dtBufferFirst)     // there are ticks in the buffer, but history and buffer do not intersect => add the whole history
                        {
                            bAddWholeHist = true;
                        }
                    }
                }
                if (bAddWholeHist)
                {
                    tickAsk.AddLastTicks(tickAskHist);
                    tickBid.AddLastTicks(tickBidHist);
                }
                else
                {
                    dtBufferFirst = tickAskBuffer.getDate(0);
                    for (int i = 0; i < tickAskHist.size(); i++)
                    {
                        DateTime dtHist = tickAskHist.getDate(i);
                        if (dtHist < dtBufferFirst)                             // add history ticks until the beginning if the buffer
                        {
                            tickAsk.AddLast(dtHist, tickAskHist.getTick(i));
                            tickBid.AddLast(dtHist, tickBidHist.getTick(i));
                        }
                        else if (dtHist == dtBufferFirst)                       // check whether we found start of the intersection (tickHist and tickBuffer)
                        {                                                       // only add history ticks if we didn't reach the start of the buffer
                            bool bSame = IsSamePatternStarted(i, tickBidHist, tickBidBuffer, tickAskHist, tickAskBuffer);
                            if (bSame)
                            {
                                break;
                            }
                            else // even the date is the same, pattern have not been started
                            {
                                tickAsk.AddLast(dtHist, tickAskHist.getTick(i));
                                tickBid.AddLast(dtHist, tickBidHist.getTick(i));
                            }
                        }
                        else if (dtHist > dtBufferFirst)
                        {
                            break;
                        }
                    }
                }
                tickAsk.AddLastTicks(tickAskBuffer); // add buffer to the end
                tickBid.AddLastTicks(tickBidBuffer); // add buffer to the end

            }
            catch (Exception e)
            {
                Console.WriteLine("Exception occured in MergeTicksWithBuffer function; {0}", e.ToString());
            }
        }
        public static void ProcessPendingEvents(IntPtr stratInstance, string sInstrument, out bool bNewPrices)
        {
            //find new ticks while looking for update of the "offer" table
            bNewPrices = false;
            FXCore.PendingEventsMonitorAut monitor = (FXCore.PendingEventsMonitorAut)Program.TradeDesk.PendingEventsMonitor;
            FXCore.TradeDeskEventEnumAut pending_events = (FXCore.TradeDeskEventEnumAut)Program.TradeDesk.GetPendingEvents();

            if (pending_events.Count > 0)
            {
                foreach (FXCore.TradeDeskEventAut pending_event in pending_events)
                {
                    if (pending_event.Kind == pending_event.KIND_CHANGE)
                    {
                        FXCore.TableAut aTable = (FXCore.TableAut)pending_event.Table;
                        if ((aTable != null) && (aTable.Type == "offers"))      // "tick" event
                        {
                            string rowId = pending_event.RowID;
                            FXCore.IRowAut row = (FXCore.IRowAut)aTable.FindRow("OfferID", rowId, 0);
                            if (row != null)
                            {
                                string sRowInstr = row.CellValue("Instrument").ToString();
                                if (sRowInstr == sInstrument)
                                {
                                    string strQuoteID = row.CellValue("QuoteID").ToString();
                                    if (strQuoteID != sLastQuoteID) // new prices
                                    {
                                        sLastQuoteID = strQuoteID;
                                        bNewPrices = true;
                                        string strBid = row.CellValue("Bid").ToString();
                                        double dblBid = Double.Parse(strBid);
                                        string strAsk = row.CellValue("Ask").ToString();
                                        double dblAsk = Double.Parse(strAsk);
                                        string strDateTime = row.CellValue("Time").ToString();
                                        DateTime dt = Convert.ToDateTime(strDateTime);
                                        FXCore.TimeZoneConverterAut timeZoneCoverter = (FXCore.TimeZoneConverterAut)Program.TradeDesk.TimeZoneConverter;
                                        DateTime dtLocal = timeZoneCoverter.Convert(dt, timeZoneCoverter.ZONE_UTC, timeZoneCoverter.ZONE_LOCAL);
                                        Console.WriteLine("DATE: {0} BID: {1} ASK: {2}", dtLocal.ToString(), dblBid, dblAsk);
                                        ProcessTick(stratInstance, sInstrument, dtLocal, dblBid, dblAsk);
                                    }
                                }
                            }
                        }
                    }
                }
            }
            else
            {
                monitor.Wait(1000);
            }
        }
        private static void ProcessTick(IntPtr stratInstance, string sInstrument, DateTime dtLocal, double dblBid, double dblAsk)  // Got tick? Add it to streams
        {
            List<Ticket> tkts;
            if (dictHistoryStreams.TryGetValue(stratInstance, out tkts))
            {
                foreach (Ticket tkt in tkts)
                {
                    if (tkt.TillNow)
                    {
                        if (tkt.Src.getInstrument() == sInstrument)
                        {
                            double dblTick;
                            if (tkt.Src.isBid())
                                dblTick = dblBid;
                            else
                                dblTick = dblAsk;

                            if (tkt.HistCompleted)
                            {
                                AddTick2Stream(dtLocal, dblTick, tkt.Src);
                            }
                            else // not ready yet? keep ticks for later use
                            {
                                if (tkt.Src.isBid())
                                    tkt.TickBuffer.AddLast(dtLocal, dblBid);
                                else
                                    tkt.TickBuffer.AddLast(dtLocal, dblAsk);
                            }
                        }
                    }
                }
            }
        }
        public static void AddTick2Stream(DateTime dt, double dblTickVal, IndicatorSource src) // adds tick value to either tick stream or to bar stream
        {
            try
            {
                if (src.isBar())
                {
                    MyIndicatorBarSource barHist = (MyIndicatorBarSource)src;
                    bool bStartNewBar = false; // continue previous bar or start a new bar?
                    object candleStart;
                    object candleEnd;
                    FXCore.PriceHistoryHelperAut priceHistHelper = Order2Go.PriceHistHelper;
                    if (src.size() == 0)
                    {
                        bStartNewBar = true;
                    }
                    else
                    {
                        DateTime dtLast = src.date(src.size() - 1);
                        priceHistHelper.GetCandle(dtLast, barHist.getBarSize(), Order2Go.GetTradingDayOffset(), Order2Go.GetTradingWeekOffset(), out candleStart, out candleEnd);
                        DateTime dtCandleEnd = Convert.ToDateTime(candleEnd);
                        if (dt >= dtCandleEnd)
                        {
                            bStartNewBar = true;
                        }
                    }

                    if (bStartNewBar)
                    {
                        priceHistHelper.GetCandle(dt, barHist.getBarSize(), Order2Go.GetTradingDayOffset(), Order2Go.GetTradingWeekOffset(), out candleStart, out candleEnd);
                        DateTime dtStartBar = Convert.ToDateTime(candleStart);
                        barHist.AddLast(dtStartBar, dblTickVal, dblTickVal, dblTickVal, dblTickVal);
                    }
                    else
                    {
                        int iLastIndex = barHist.size() - 1;
                        DateTime dtStartBar = barHist.date(iLastIndex);
                        double dblOldOpenVal = barHist.getOpen(iLastIndex);
                        double dbOldHighVal = barHist.getHigh(iLastIndex);
                        double dblOldLowVal = barHist.getLow(iLastIndex);
                        barHist.RemoveLast();
                        double dblNewOpenVal = dblOldOpenVal;
                        double dblNewHighVal = dbOldHighVal;
                        double dblNewLowVal = dblOldLowVal;
                        double dblNewCloseVal = dblTickVal;

                        if (dblTickVal > dbOldHighVal)
                        {
                            dblNewHighVal = dblTickVal;
                        }
                        if (dblTickVal < dblNewLowVal)
                        {
                            dblNewLowVal = dblTickVal;
                        }
                        barHist.AddLast(dtStartBar, dblNewOpenVal, dblNewHighVal, dblNewLowVal, dblNewCloseVal);
                    }

                }
                else // tickHist
                {
                    MyIndicatorTickSource tickHist = (MyIndicatorTickSource)src;
                    tickHist.AddLast(dt, dblTickVal);
                }
            }
            catch (Exception e)
            {
                Console.WriteLine("Exception occured in AddTick2Stream: {0}", e.ToString());
            }
        }
    }
}
