using System;
using System.Collections.Generic;
using System.Text;
using gehtsoft.indicore;

namespace ApplyStrategy
{
    class MyHost : Host
    {
        public string version = "SomeVersion";
        public override string getVersion()
        {
            return version;
        }
        public override TradingDataTable findTable(string table)
        {
            try
            {
                MyTradingTable tradingTable = new MyTradingTable(table);
                return tradingTable;
            }
            catch
            {
                return null;
            }
        }
        public override bool execute(IntPtr instance, string text, List<HostArg> arglist, HostArg res)
        {
            bool bReturn = false;
            switch (text)
            {
                case "getTradingDayOffset":
                    res.setNumber(Order2Go.GetTradingDayOffset());
                    bReturn = true;
                    break;
                case "getTradingWeekOffset":
                    res.setNumber(Order2Go.GetTradingWeekOffset());
                    bReturn = true;
                    break;
                case "getTradingProperty":
                    Order2Go.GetTradingProperty(arglist, ref res);
                    if (res.getType() != HostArg.Type.Error)
                        bReturn = true;
                    break;
                case "getHistory":  //host:execute("getHistory", 1, source:instrument(), "D1", from , to, source:isBid());
                    if (arglist.Count < 6)
                    {
                        Console.WriteLine("Wrong number of arguments in 'getHistory'");
                        bReturn = false;
                    }
                    else
                    {
                        double cookie = arglist[0].getNumber();
                        string instrument = arglist[1].getString();
                        string barSize = arglist[2].getString();
                        double from = arglist[3].getNumber();
                        double to = arglist[4].getNumber();
                        bool isBid = arglist[5].getBool();
                        IndicatorSource bidSrc;
                        IndicatorSource askSrc;
                        HistoryMgr.GetPriceHistory(instance, instrument, barSize, DateTime.FromOADate(from), DateTime.FromOADate(to), 0, isBid, !isBid, (int)cookie, out bidSrc, out askSrc);
                        IndicatorSource srcStream;
                        if (isBid)
                            srcStream = bidSrc;
                        else // isAsk
                            srcStream = askSrc;

                        IntPtr anIntPtr = srcStream.InternalID;
                        res.setSource(anIntPtr);
                        bReturn = true;
                    }
                    break;
                case "extendHistory": //string host:execute ("extendHistory", cookie, stream, from, to)
                    bReturn = true;
                    res.setError("Method 'extendHistory' has to be implemented soon");
                    throw new Exception("The method or operation is not implemented.");
                case "stop":
                    Program.Stopped = true;
                    bReturn = true;
                    break;
                case "isTableFilled":
                    if (arglist.Count < 1)
                    {
                        Console.WriteLine("Wrong number of arguments in 'isTableFilled'");
                        bReturn = false;
                    }
                    else
                    {
                        bool bIsFilled = false;
                        bool bRightTableType = false;
                        string sTable = arglist[0].getString();
                        switch (sTable)
                        {
                            case "offers":
                            case "summary":
                            case "accounts":
                            case "orders":
                            case "trades":
                            case "closed trades":
                            case "messages":
                                bRightTableType = true;
                                break;
                            default:
                                break;
                        }
                        if (bRightTableType)
                        {
                            FXCore.TableAut O2GoTable = (FXCore.TableAut)Program.TradeDesk.FindMainTable(sTable);
                            if (O2GoTable != null)
                            {
                                string sStatus = O2GoTable.Status;
                                if (String.Equals(sStatus, "refreshed"))
                                    bIsFilled = true;
                            }
                        }
                        res.setBool(bIsFilled);
                        bReturn = true;
                    }
                    break;
                case "convertTime": // string host:execute ("convertTime", tzFrom, tzTo, date)
                    if (arglist.Count < 3)
                    {
                        Console.WriteLine("Wrong number of arguments in 'convertTime'");
                        bReturn = false;
                    }
                    else
                    {
                        double fromTimeZone = arglist[0].getNumber();
                        double toTimeZone = arglist[1].getNumber();
                        double dblInputDate = arglist[2].getNumber();
                        double dblOutputDate = convertTime((int)fromTimeZone, (int)toTimeZone, dblInputDate);
                        res.setNumber(dblOutputDate);
                        bReturn = true;
                    }
                    break;
                case "setTimer":    // timerid(number) = host:execute("setTimer", cookie, timeout)
                    if (arglist.Count < 2)
                    {
                        Console.WriteLine("Wrong number of arguments in 'setTimer'");
                        bReturn = false;
                    }
                    else
                    {
                        int iCookie = (int)arglist[0].getNumber();
                        int iTimeout = (int)arglist[1].getNumber();
                        int iTimerID = TimersMgr.AddTimer(iCookie, iTimeout);
                        res.setNumber(iTimerID);
                        bReturn = true;
                    }
                    break;
                case "killTimer":   // host:execute("killTimer, timerid);
                    if (arglist.Count < 1)
                    {
                        Console.WriteLine("Wrong number of arguments in 'killTimer'");
                        bReturn = false;
                    }
                    else
                    {
                        int iTimerID = (int)arglist[0].getNumber();
                        TimersMgr.KillTimer(iTimerID);
                        bReturn = true;
                    }
                    break;
                default:
                    Console.WriteLine("This command is not implemented in execute");
                    bReturn = false;
                    break;
            }
            return bReturn;
        }
        public override void trace(IntPtr instance, string text)
        {
            Console.WriteLine(text);
        }

private double convertTime(int from, int to, double date) // Convert time from one time zone to another
{
    DateTime dtInput = DateTime.FromOADate(date);
    DateTime dtOutput = dtInput;
    DateTime dtLocal = dtInput;
    FXCore.TimeZoneConverterAut timeConverter = Order2Go.TimeZoneConverter;
    int iTradingDayOffset = Order2Go.GetTradingDayOffset();

    if (from == (int)TimeZones.TZ_TS)
        from = (int)TimeZones.TZ_LOC;
    if (to == (int)TimeZones.TZ_TS)
        to = (int)TimeZones.TZ_LOC;

    if (from == to)
        return date;

    if (from == (int)TimeZones.TZ_UTC)
    {
        switch (to)
        {
            case (int)TimeZones.TZ_EST:
                dtOutput = timeConverter.Convert(dtInput, timeConverter.ZONE_UTC, timeConverter.ZONE_EST);
                break;
            case (int)TimeZones.TZ_LOC:
                dtOutput = timeConverter.Convert(dtInput, timeConverter.ZONE_UTC, timeConverter.ZONE_LOCAL);
                break;
            case (int)TimeZones.TZ_FINANCIAL:
                dtLocal = timeConverter.Convert(dtInput, timeConverter.ZONE_UTC, timeConverter.ZONE_LOCAL);
                dtOutput = dtLocal.Subtract(TimeSpan.FromHours(iTradingDayOffset));
                break;
            case (int)TimeZones.TZ_SERV:
                dtOutput = timeConverter.Convert(dtInput, timeConverter.ZONE_UTC, timeConverter.ZONE_SERVER);
                break;
        }
    }
    else if (from == (int)TimeZones.TZ_FINANCIAL)
    {
        dtLocal = dtInput.AddHours(iTradingDayOffset); 
        switch (to)
        {
            case (int)TimeZones.TZ_EST:
                dtOutput = timeConverter.Convert(dtLocal, timeConverter.ZONE_LOCAL, timeConverter.ZONE_EST);
                break;
            case (int)TimeZones.TZ_LOC:
                dtOutput = dtLocal; 
                break;
            case (int)TimeZones.TZ_SERV:
                dtOutput = timeConverter.Convert(dtLocal, timeConverter.ZONE_LOCAL, timeConverter.ZONE_SERVER);
                break;
            case (int)TimeZones.TZ_UTC:
                dtOutput = timeConverter.Convert(dtLocal, timeConverter.ZONE_LOCAL, timeConverter.ZONE_UTC);
                break;
        }
    }
    else if (from == (int)TimeZones.TZ_EST)
    {
        switch (to)
        {
            case (int)TimeZones.TZ_FINANCIAL:
                dtLocal = timeConverter.Convert(dtInput, timeConverter.ZONE_EST, timeConverter.ZONE_LOCAL);
                dtOutput = dtLocal.Subtract(TimeSpan.FromHours(iTradingDayOffset));
                break;
            case (int)TimeZones.TZ_LOC:
                dtOutput = timeConverter.Convert(dtInput, timeConverter.ZONE_EST, timeConverter.ZONE_LOCAL);
                break;
            case (int)TimeZones.TZ_SERV:
                dtOutput = timeConverter.Convert(dtInput, timeConverter.ZONE_EST, timeConverter.ZONE_SERVER);
                break;
            case (int)TimeZones.TZ_UTC:
                dtOutput = timeConverter.Convert(dtInput, timeConverter.ZONE_EST, timeConverter.ZONE_UTC);
                break;
        }
    }
    else if (from == (int)TimeZones.TZ_LOC)
    {
        switch (to)
        {
            case (int)TimeZones.TZ_EST:
                dtOutput = timeConverter.Convert(dtInput, timeConverter.ZONE_LOCAL, timeConverter.ZONE_EST);
                break;
            case (int)TimeZones.TZ_FINANCIAL:
                dtOutput = dtInput.Subtract(TimeSpan.FromHours(iTradingDayOffset)); 
                break;
            case (int)TimeZones.TZ_SERV:
                dtOutput = timeConverter.Convert(dtInput, timeConverter.ZONE_LOCAL, timeConverter.ZONE_SERVER);
                break;
            case (int)TimeZones.TZ_UTC:
                dtOutput = timeConverter.Convert(dtInput, timeConverter.ZONE_LOCAL, timeConverter.ZONE_UTC);
                break;
        }
    }
    else if (from == (int)TimeZones.TZ_SERV)
    {
        switch (to)
        {
            case (int)TimeZones.TZ_EST:
                dtOutput = timeConverter.Convert(dtInput, timeConverter.ZONE_SERVER, timeConverter.ZONE_EST);
                break;
            case (int)TimeZones.TZ_FINANCIAL:
                dtLocal = timeConverter.Convert(dtInput, timeConverter.ZONE_SERVER, timeConverter.ZONE_LOCAL);
                dtOutput = dtLocal.Subtract(TimeSpan.FromHours(iTradingDayOffset));
                break;
            case (int)TimeZones.TZ_LOC:
                dtOutput = timeConverter.Convert(dtInput, timeConverter.ZONE_SERVER, timeConverter.ZONE_LOCAL);
                break;
            case (int)TimeZones.TZ_UTC:
                dtOutput = timeConverter.Convert(dtInput, timeConverter.ZONE_SERVER, timeConverter.ZONE_UTC);
                break;
        }
    }
    double dblOutput = dtOutput.ToOADate(); 
    return dblOutput;
}
    }

    enum TimeZones
    {
        TZ_EST = 1,
        TZ_UTC,
        TZ_LOC,
        TZ_TS,
        TZ_SERV,
        TZ_FINANCIAL
    }
}
