using System;
using System.Collections.Generic;
using System.Text;
using gehtsoft.indicore;

namespace ApplyStrategy
{
    class MyIndicatorBarSource : IndicatorBarSource
    {
        public MyIndicatorBarSource() { }
        public MyIndicatorBarSource(bool bIsBid, string sInstrument)
        {
            setInstrument(sInstrument);
            setBid(bIsBid);
            setPrecision(5);
            setPipSize(0.0001);
        }
        private List<BarData> myData = new List<BarData>();
        private int precision = 4;
        private double pipSize = 0.01;
        private string instrument;
        private string name = "barName";
        private string barSize = "m1";
        private bool bIsBid = true;

        private int mSerial = 0;

        public void AddLast(System.DateTime dt, double open, double high, double low, double close)
        {
            BarData bd = new BarData();
            bd.serial = ++mSerial;
            bd.dt = dt;
            bd.highVal = high;
            bd.lowVal = low;
            bd.openVal = open;
            bd.closeVal = close;
            myData.Add(bd);
        }

        public void RemoveLast()
        {
            myData.RemoveAt(myData.Count - 1);
            mSerial--;
        }

        public override DateTime date(int index)
        {
            DateTime dt = myData[index].dt;
            return dt;
        }
        public override string getBarSize()
        {
            return barSize;
        }
        public void setBarSize(string barSizeVal)
        {
            this.barSize = barSizeVal;
        }
        public override double getClose(int index)
        {
            BarData barD = myData[index];
            double close = barD.closeVal;
            return close;
        }
        public override int getDisplayPrecision()
        {
            return precision;
        }
        public override int getFirstPeriod()
        {
            return 0;
        }
        public override double getHigh(int index)
        {
            BarData barD = myData[index];
            double high = barD.highVal;
            return high;
        }
        public override string getInstrument()
        {
            return instrument;
        }
        public void setInstrument(string instrumentVal)
        {
            this.instrument = instrumentVal;
        }
        public override double getLow(int index)
        {
            BarData barD = myData[index];
            double low = barD.lowVal;
            return low;
        }
        public override string getName()
        {
            return name;
        }
        public void setName(string aName)
        {
            this.name = aName;
        }
        public override double getOpen(int index)
        {
            BarData barD = myData[index];
            double open = barD.openVal;
            return open;
        }
        public override double getPipSize()
        {
            return pipSize;
        }
        public void setPipSize(double pipSizeVal)
        {
            this.pipSize = pipSizeVal;
        }
        public override int getPrecision()
        {
            return precision;
        }
        public void setPrecision(int precisionVal)
        {
            this.precision = precisionVal;
        }
        public override int getSerial(int index)
        {
            return myData[index].serial;
        }
        public override double getVolume(int index)
        {
            throw new Exception("The method or operation is not implemented.");
        }
        public override bool hasData(int index)
        {
            return true; // unless it is a result of calculations
        }
        public override bool isAlive()
        {
            return true;
        }
        public override bool isBar()
        {
            return true;
        }
        public override bool isBid()
        {
            return bIsBid;
        }
        public void setBid(bool bIsBidVal)
        {
            this.bIsBid = bIsBidVal;
        }
        public override int size()
        {
            return myData.Count;
        }
        public override bool supportsVolume()
        {
            return false; // do not support Volume so far
        }
    }
    class BarData
    {
        public int serial;
        public DateTime dt;
        public double highVal;
        public double lowVal;
        public double openVal;
        public double closeVal;
        public void Clear()
        {
            serial = 0;
            dt = DateTime.MinValue;
            highVal = 0;
            lowVal = 0;
            openVal = 0;
            closeVal = 0;
            return;
        }
    }
}
