using System;
using System.Collections.Generic;
using System.Text;
using gehtsoft.indicore;
using System.Net.Mail;

namespace ApplyStrategy
{
    class MyStrategyCallBack : StrategyCallback
    {
        public override void alertEmail(IntPtr instance, string to, string subject, string text)
        {
            MailMessage message = new MailMessage();
            SmtpClient client = new SmtpClient();
            // TODO: *** Set "from" address ***
            message.From = new MailAddress("fromSender@domain.com");
            message.To.Add(new MailAddress(to));
            message.Subject = subject;
            message.Body = text;
            // TODO: *** Modify for your SMTP server ***
            client.Host = "mail.optonline.net";
            client.Send(message);
        }
        public override void alertMessage(IntPtr instance, string instrument, double price, string signalname, DateTime time)
        {
            Console.WriteLine("Instrument: {0}, price: {1}, signalname: {2}, date/time: {3}", instrument, price, signalname, time.ToString());
        }
        public override void alertSound(IntPtr instance, string file, bool recurrent)
        {
            throw new Exception("The method or operation is not implemented.");
        }
        public override string executeOrder(IntPtr instance, int cookie, ValueMap map, out string error)
        { // function returns request ID or "" if order cannot be executed
            string strRet = string.Empty;
            object objRequestID = null;
            try
            {
                error = string.Empty;
                // The folowing 5 parameters are always required
                string strAcctID = GetStringParamFromValueMap(map, "AcctID");
                string strOfferID = GetStringParamFromValueMap(map, "OfferID");
                int iQuantity = (int)GetNumberParamFromValueMap(map, "Quantity");
                string strBuySell = GetStringParamFromValueMap(map, "BuySell");         // "B" or "S"
                string strOrderType = GetStringParamFromValueMap(map, "OrderType");     // "OM" "O" "OR" "CM" "C" "CR" "SE" "LE" "S" "L" 

                string strNetQtyFlag = GetStringParamFromValueMap(map, "NetQtyFlag");   // "y" or "n"(default) - close all positions for account and instrument in the direction ("BuySell") (for "C", "CM", "CR") or opposite to direction (for "SE" and "LE")
                string strTradeID = GetStringParamFromValueMap(map, "TradeID");         // For "C", "CM", "CR", "S", and "L" ("all" for new quantity order)
                double dblRate = 0;
                if (strOrderType != "OM" && strOrderType != "CM")
                    dblRate = GetNumberParamFromValueMap(map, "Rate");

                // for "OR" and "CR" either ("RateMin" and "RateMax") or a ("Rate" and "AtMarket") must be specified.
                double dblRateMin = GetNumberParamFromValueMap(map, "RateMin"); 
                double dblRateMax = GetNumberParamFromValueMap(map, "RateMax"); 
                int iAtMarket = (int)GetNumberParamFromValueMap(map, "AtMarket");       // distance from "Rate" to min and max prices (in pips)

                string strGTC = GetStringParamFromValueMap(map, "GTC");                 // "GTC" (Good-Till-Cancelled), "IOC" (Immediate-or-Cancel), or "FOK" (Fill-or-Kill).
                double dblTrailUpdatePips = GetNumberParamFromValueMap(map, "TrailUpdatePips"); // for "S" and "SE": trailing folowing market; max change of the price after which rate of order would be changed as well (in "Digits"); 1 - dynamic trailiing, 0 - no trailing

                // for "S" and "L" either "Rate" or ("PegType" and PegPriceOffsetPips") must be specified
                string strPegType = GetStringParamFromValueMap(map, "PegType");         // pegged againt price: "O" (open) or "M" (current market)
                double dblPegPriceOffsetPips = GetNumberParamFromValueMap(map, "PegPriceOffsetPips"); // offset to the price specified in the PegType

                string strCustomID = GetStringParamFromValueMap(map, "CustomID");       // custom identifier of the order (for order-related QTXT columns of "Orders", "Trades", and "Closed Trades")

                //Orders which create a new position (OM, O, OR, SE and LE orders) may be associated with stop and limit orders
                    // either RateStop or a pair of PegPriceOffsetPipsStop and PegTypeStop must be used
                double dblRateStop = GetNumberParamFromValueMap(map, "RateStop");
                double dblPegPriceOffsetPipsStop = GetNumberParamFromValueMap(map, "PegPriceOffsetPipsStop");
                string strPegTypeStop = GetStringParamFromValueMap(map, "PegTypeStop");  // "O" or "M"
                int iTrailStepStop = (int)GetNumberParamFromValueMap(map, "TrailStepStop"); // for "O", "OM, "OR", "SE", and "LE": folowing market (in "Digits"); 0 - no trailing; 1 - dynamic trailing
                    // either RateLimit or a pair of PegPriceOffsetPipsLimit and PegTypeLimit fields must be used 
                double dblRateLimit = GetNumberParamFromValueMap(map, "RateLimit");
                double dblPegPriceOffsetPipsLimit = GetNumberParamFromValueMap(map, "PegPriceOffsetPipsLimit");
                string strPegTypeLimit = GetStringParamFromValueMap(map, "PegTypeLimit");

                string sInstrument = (string)GetValueFromTable("offers", strOfferID, "Instrument", "String");
                string strQuoteID = string.Empty;
                if (dblRate != 0)
                    strQuoteID = (string)GetValueFromTable("offers", strOfferID, "QuoteID", "String");

                bool bBuy = true;
                if (strBuySell == "S")
                    bBuy = false;

                int iTIFType = GetTimeInForce(strGTC); // time in force
                if (strOrderType == "OR" || strOrderType == "CR")
                {
                    if (iAtMarket == 0)
                        GetValuesForRangeOrder(ref dblRate, ref iAtMarket, dblRateMin, dblRateMax, strOfferID);
                }

                double dblUseRateStop = dblRateStop;    // depending on "Stop" type either RateStop or offset for the "Stop" will be used
                double dblUseRateLimit = dblRateLimit;  // depending on "Limit" type either RateLimit or offset for the "Limit" will be used
                int iSLType = GetStopLimitType(strOrderType,
                    dblRate, strPegType, dblPegPriceOffsetPips,
                    dblRateStop, strPegTypeStop, dblPegPriceOffsetPipsStop,
                    dblRateLimit, strPegTypeLimit, dblPegPriceOffsetPipsLimit,
                    ref dblUseRateStop, ref dblUseRateLimit);

                if (strOrderType == "C" || strOrderType == "CM" || strOrderType == "CR")
                {
                    Program.TradeDesk.CloseTrade2Async(strTradeID, iQuantity, dblRate, strQuoteID, iAtMarket, iTIFType, out objRequestID);
                }

                if (strOrderType == "O" || strOrderType == "OM" || strOrderType == "OR")
                {
                    Program.TradeDesk.OpenTrade3Async(strAcctID, sInstrument, bBuy, iQuantity, dblRate, string.Empty, iAtMarket,
                        iSLType, dblUseRateStop, dblUseRateLimit, iTrailStepStop, 0, iTIFType, out objRequestID);
                }

                if (strOrderType == "SE" || strOrderType == "LE")
                {
                    Program.TradeDesk.CreateEntryOrder3Async(strAcctID, sInstrument, bBuy, iQuantity, dblRate,
                        iTrailStepStop, iSLType, dblUseRateStop, dblUseRateLimit, iTrailStepStop, 0, out objRequestID);
                }

                if (strOrderType == "S" || strOrderType == "L")
                {
                    string strOpenOrderID = (string)GetValueFromTable("trades", strTradeID, "OpenOrderID", "String");
                    Program.TradeDesk.ChangeEntryOrderStopLimit3Async(strOpenOrderID, dblRate, iSLType, iTrailStepStop, out objRequestID);
                }

                if (objRequestID != null)
                {
                    strRet = (string)objRequestID;
                    Order2Go.Attach2Instance(instance, strRet, cookie, strOrderType);
                }
                return strRet;
            }
            catch (Exception e)
            {
                error = e.ToString();
                Console.WriteLine("Exception on 'executeOrder' {0}", error);
                string sNotifyError;
                Program.Instance.notifyFinished(cookie, false, e.Message, out sNotifyError);
                if (!string.IsNullOrEmpty(sNotifyError))
                    Console.WriteLine("Error on notifyFinished: {0}", sNotifyError);
               
                return strRet;
            }
        }
        private string GetStringParamFromValueMap(ValueMap map, string strParamName) // get string parameter from ValueMap
        {
            string strParam = string.Empty;
            try
            {
                HostArg hostArgParam = map.get(strParamName);
                if (!(hostArgParam.isNil()))
                    strParam = hostArgParam.getString();
                return strParam;
            }
            catch
            {
                return strParam;
            }
        }
        private double GetNumberParamFromValueMap(ValueMap map, string strParamName) // get number parameter from ValueMap
        {
            double dblParam = 0;
            try
            {
                HostArg hostArgParam = map.get(strParamName);
                if (!(hostArgParam.isNil()))
                    dblParam = hostArgParam.getNumber();
                return dblParam;
            }
            catch
            {
                return dblParam;
            }
        }
        private object GetValueFromTable(string strTable, string strPrimaryKeyValue, string strColumnName, string strColumnType)
        {
            // get the value of some column of the table, when primary key = specific value 
            // for example, get "Instrument" or "QuoteID" or "PointSize" from "offers" table when "OfferID" = strPrimaryKeyValue
            HostArg hostArgValue = new HostArg();
            hostArgValue.setNil();
            try
            {
                MyTradingTable aTable = (MyTradingTable)Program.Host.findTable(strTable);
                if (aTable != null)
                {
                    HostArg hostArgOfferId = new HostArg();
                    hostArgOfferId.setString(strPrimaryKeyValue);
                    string strPrimaryKey = Order2Go.GetPrimaryKey(strTable);
                    MyTradingRow aRow = (MyTradingRow)aTable.findRow(strPrimaryKey, hostArgOfferId, 0); // find the row where primary key = primary key value
                    if (aRow != null)
                    {
                        hostArgValue = aRow.cellValue(strColumnName);
                        if (strColumnType == "String")
                            return hostArgValue.getString();
                        else if (strColumnType == "Double")
                            return  hostArgValue.getNumber();
                        else if (strColumnType == "Integer")
                            return (int)hostArgValue.getNumber(); 
                        else if (strColumnType == "DateTime")
                            return Convert.ToDateTime(hostArgValue.getString());
                    }
                }
                return hostArgValue;
            }
            catch
            {
                hostArgValue.setNil();
                return hostArgValue;
            }
        }
        private int GetTimeInForce(string strGTC) // get "time in force" integer according to the string "GTC" (good till cancel) value 
        {
            int iTIF = 0;
            switch (strGTC)
            {
                case "GTC": // good till cancel
                    iTIF = Program.TradeDesk.TIF_GTC;
                    break;
                case "IOC": // immidiate or cancel
                    iTIF = Program.TradeDesk.TIF_IOC;
                    break;
                case "FOK": // fill or kill
                    iTIF = Program.TradeDesk.TIF_FOK;
                    break;
                default:
                    break;
            }
            return iTIF;
        }
        private int GetStopLimitType(string strOrderType,
            double dblRate, string strPegType, double dblPegPriceOffsetPips,
            double dblRateStop, string strPegTypeStop, double dblPegPriceOffsetPipsStop,
            double dblRateLimit, string strPegTypeLimit, double dblPegPriceOffsetPipsLimit,
                    ref double dblUseRateStop, ref double dblUseRateLimit)
        // 1 - get the type of "Stop" order (or "Limit" order)
        // 2 - set the values to be used as "RateStop" and "RateLimit" depending on "StopLimit" value (for "pegged" orders use offset in pips instead of "real" value)
        {
            // For "OM", "O", "OR", "SE", "LE" either the RateStop(RateLimit) or 
            //      a pair of (PegTypeStop and PegPriceOffsetPipsStop) (PegTypeLimit and PegPriceOffsetPipsLimit) must be specified 
            // For "S" and "L":
            //      Rate for existing position and 
            //      either Rate or (PegType and PegPriceOffsetPips) must be specified if "Stop" or "Limit" order was created for an entry order
            int iSLType = Program.TradeDesk.SL_NONE;

            switch (strOrderType)
            {
                case "S":
                    if (dblRate != 0)
                        iSLType = iSLType + Program.TradeDesk.SL_STOP;
                    else
                    {
                        if (dblPegPriceOffsetPips != 0)
                        {
                            dblUseRateStop = dblPegPriceOffsetPips;
                            if (strPegType == "O")
                                iSLType = iSLType + Program.TradeDesk.SL_PEGSTOPOPEN;
                            else if (strPegType == "M") // "Close" ("Market") price
                                iSLType = iSLType + Program.TradeDesk.SL_PEGSTOPCLOSE;
                        }
                    }
                    break;
                case "OM":
                case "O":
                case "OR":
                case "SE":
                case "LE":
                    if (dblRateStop != 0)
                        iSLType = iSLType + Program.TradeDesk.SL_STOP;
                    else if (dblPegPriceOffsetPipsStop != 0)
                    {
                        dblUseRateStop = dblPegPriceOffsetPipsStop;
                        if (strPegTypeStop == "O")
                            iSLType = iSLType + Program.TradeDesk.SL_PEGSTOPOPEN;
                        else if (strPegTypeStop == "M")
                            iSLType = iSLType + Program.TradeDesk.SL_PEGSTOPCLOSE;
                    }
                    break;
                default:
                    break;
            }

            switch (strOrderType)
            {
                case "L":
                    if (dblRate != 0)
                        iSLType = iSLType + Program.TradeDesk.SL_LIMIT;
                    else
                    {
                        if (dblPegPriceOffsetPips != 0)
                        {
                            dblUseRateLimit = dblPegPriceOffsetPips;
                            if (strPegType == "O")
                                iSLType = iSLType + Program.TradeDesk.SL_PEGLIMITOPEN;
                            else if (strPegType == "M")
                                iSLType = iSLType + Program.TradeDesk.SL_PEGLIMITCLOSE;
                        }
                    }
                    break;
                case "OM":
                case "O":
                case "OR":
                case "SE":
                case "LE":
                    if (dblRateLimit != 0)
                        iSLType = iSLType + Program.TradeDesk.SL_LIMIT;
                    else if (dblPegPriceOffsetPipsLimit != 0)
                    {
                        dblUseRateLimit = dblPegPriceOffsetPipsLimit;
                        if (strPegTypeLimit == "O")
                            iSLType = iSLType + Program.TradeDesk.SL_PEGLIMITOPEN;
                        else if (strPegTypeLimit == "M")
                            iSLType = iSLType + Program.TradeDesk.SL_PEGLIMITCLOSE;
                    }
                    break;
                default:
                    break;
            }
            return iSLType;
        }       
        private void GetValuesForRangeOrder(ref double dblRate, ref int iAtMarket, double dblRateMin, double dblRateMax, string strOfferID)
        {
            // if we have RateMin and RateMax, convert them into Rate and iAtMarket for "Range" orders ("OR" or "CR")
            if ((dblRateMin != 0) && (dblRateMax != 0))
            {
                double dblPointSize = (double)GetValueFromTable("offers", strOfferID, "PointSize", "Double");
                double dblNumPointsInRange = (dblRateMax - dblRateMin) / dblPointSize;
                dblRate = (dblRateMax + dblRateMin) / 2;
                iAtMarket = (int)(dblNumPointsInRange / 2 + 0.5); // need to add 0.5; otherwise casting to int drops any decimal places (without rounding) 
            }
        }
    }
}
