
using System;
using System.Collections.Generic;
using System.Text;
using gehtsoft.indicore;

namespace ApplyStrategy
{
    public class MyTradingTable : TradingDataTable
    {
        private string sTableName;
        public string TableName
        {
            get { return sTableName; }
        }
        private TradingDataTableEnumerator rowEnum;
        private List<MyTradingRow> lstRows = new List<MyTradingRow>();
        public List<MyTradingRow> Rows
        {
            get { return lstRows; }
        }
        public MyTradingTable(string strTable)
        {
            FXCore.TableAut O2GoTable = (FXCore.TableAut)Program.TradeDesk.FindMainTable(strTable);
            if (O2GoTable != null)
            {
                sTableName = strTable;
                FXCore.ColumnsEnumAut columns = (FXCore.ColumnsEnumAut)O2GoTable.Columns;
                FXCore.RowsEnumAut lstRows = (FXCore.RowsEnumAut)O2GoTable.Rows;
                foreach (FXCore.RowAut row in lstRows)
                {
                    MyTradingRow aRow = new MyTradingRow(strTable);
                    foreach (FXCore.ColumnAut column in columns)
                    {
                        try
                        {
                            object objVal = row.CellValue(column.ID);
                            string strType = objVal.GetType().Name;
                            bool bHostArgSet = false;
                            HostArg aHostArg = new HostArg();
                            if (strType.ToLower().Equals("string"))
                            {
                                aHostArg.setString(Convert.ToString(objVal));
                                bHostArgSet = true;
                            }
                            else if ((strType.ToLower().Contains("int")) || (strType.ToLower().Equals("double")))
                            {
                                aHostArg.setNumber(Convert.ToDouble(objVal));
                                bHostArgSet = true;
                            }
                            else if (strType.ToLower().Equals("boolean"))
                            {
                                aHostArg.setBool(Convert.ToBoolean(objVal));
                                bHostArgSet = true;
                            }
                            else if (strType.ToLower().Equals("datetime"))
                            {
                                DateTime dtVal = Convert.ToDateTime(objVal);
                                double dblVal = dtVal.ToOADate();
                                aHostArg.setNumber(dblVal);
                                bHostArgSet = true;
                            }

                            if (bHostArgSet)
                            {
                                aRow.Cells.Add(column.ID, aHostArg);
                            }
                        }
                        catch (InvalidCastException)
                        {
                            Console.WriteLine("Problem with column {0}", column.ID);
                        }
                    }
                    this.addRow(aRow);
                }
            }
            rowEnum = new MyTradingRowEnumerator(this);
        }
        public override TradingDataTableRow findRow(string column, HostArg key, int occurrence)
        {
            int iNumberFound = 0;
            try
            {
                for (int i = 0; i < lstRows.Count; i++)
                {
                    MyTradingRow aRow = lstRows[i];
                    if (aRow.Cells.ContainsKey(column))
                    {
                        HostArg cellValue;
                        aRow.Cells.TryGetValue(column, out cellValue);
                        int iRet = cellValue.CompareTo(key);
                        if (iRet == 0)
                        {
                            if (iNumberFound == occurrence)
                                return aRow;
                            else
                                iNumberFound++;
                        }
                    }
                }
                return null;
            }
            catch
            {
                return null;
            }
        }
        public override TradingDataTableEnumerator getEnumerator()
        {
            return rowEnum;
        }
        public void addRow(MyTradingRow aRow)
        {
            lstRows.Add(aRow);
        }
    }

    public class MyTradingRow : TradingDataTableRow
    {
        private string sTable;
        private Dictionary<string, HostArg> dictCells = new Dictionary<string, HostArg>();
        public Dictionary<string, HostArg> Cells    
        {
            get {return dictCells;}
        }
        public MyTradingRow(string sTable)
        {
            this.sTable = sTable;
        }
        public override HostArg cellValue(string column)
        {
            HostArg cellValue;
            dictCells.TryGetValue(column, out cellValue);
            return cellValue;
            
        }
        public override bool refresh()
        {
            bool bSuccess = false;
            FXCore.TableAut O2GoTable = (FXCore.TableAut)Program.TradeDesk.FindMainTable(sTable);
            if (O2GoTable.Status != "refreshed") // for example, after logoff
            {
                this.dictCells.Clear();
                return false;
            }
            else
            {
                MyTradingTable aTable = new MyTradingTable(sTable);           // load a new (refreshed) table containing a row
                if (aTable != null)
                {
                    string strPrimaryKey = Order2Go.GetPrimaryKey(sTable);    // get a primary key for the table            
                    HostArg primaryKeyValue;
                    dictCells.TryGetValue(strPrimaryKey, out primaryKeyValue);   // get a value for the primary key in this row
                    MyTradingRow aRow = (MyTradingRow)aTable.findRow(strPrimaryKey, primaryKeyValue, 0); // find a row in a refreshed table with particular value of the primary key
                    if (aRow != null)                                            // fill "old" row with refreshed values
                    {
                        this.dictCells.Clear();
                        foreach (KeyValuePair<string, HostArg> kvp in aRow.dictCells)
                        {
                            this.dictCells.Add(kvp.Key, kvp.Value);
                        }
                        bSuccess = true;
                    }
                }
                return bSuccess;
            }
        }
    }

    public class MyTradingRowEnumerator : TradingDataTableEnumerator
    {
        private MyTradingTable parentTable;
        int iRowNum = -1;
        public MyTradingRowEnumerator(MyTradingTable aTable)
        {
            parentTable = aTable;
        }
        public override TradingDataTableRow next() // point to the next row
        {
            iRowNum++;
            if (iRowNum < parentTable.Rows.Count)
                return parentTable.Rows[iRowNum];
            else
                return null;
        }
        public override bool reset()
        {
            if (parentTable == null)
            {
                return false;
            }
            else
            {
                FXCore.TableAut O2GoTable = (FXCore.TableAut)Program.TradeDesk.FindMainTable(parentTable.TableName);
                if (O2GoTable == null)
                    return false;
            }

            iRowNum = -1;
            return true;
        }
    } 
}
