using System;
using System.Collections.Generic;
using System.Text;
using gehtsoft.indicore;

namespace ApplyStrategy
{
    class Order2Go
    {
        private static Dictionary<IntPtr, List<OrderInfo>> dictOrdersInfo = new Dictionary<IntPtr, List<OrderInfo>>(); // info about strategy instance and associated orders
        public static Dictionary<IntPtr, List<OrderInfo>> OrdersInfo
        {
            get { return dictOrdersInfo; }
        }
        public static void Attach2Instance(IntPtr stratInstance, string strOrderRequest, int iCookie, string strOrderType)
        {
            // attach the order information to strategy instance (keep it in a dictionary of orders, where key is a strategy instance, and value is a list of orders with corresponding requestID and cookie)
            OrderInfo order = new OrderInfo(strOrderRequest, iCookie);
            if (dictOrdersInfo.ContainsKey(stratInstance))
            {
                List<OrderInfo> lstOrder;
                dictOrdersInfo.TryGetValue(stratInstance, out lstOrder);
                if (lstOrder != null)
                {
                    lstOrder.Add(order);
                }
            }
            else
            {
                List<OrderInfo> listOrders = new List<OrderInfo>();
                listOrders.Add(order);
                dictOrdersInfo.Add(stratInstance, listOrders);
            }
        }
        private static FXCore.PriceHistoryHelperAut priceHistHelper;
        public static FXCore.PriceHistoryHelperAut PriceHistHelper
        {
            get
            {
                if (priceHistHelper == null)
                {
                    priceHistHelper = (FXCore.PriceHistoryHelperAut)Program.TradeDesk.PriceHistoryHelper;
                }
                return priceHistHelper;
            }
        }
        private static FXCore.PermissionCheckerAut permissionsChecker;
        public static FXCore.PermissionCheckerAut PermissionsChecker
        {
            get
            {
                if (permissionsChecker == null)
                {
                    permissionsChecker = (FXCore.PermissionCheckerAut)Program.TradeDesk.PermissionChecker;
                }
                return permissionsChecker;
            }
        }
        private static FXCore.TimeZoneConverterAut timeZoneConverter;
        public static FXCore.TimeZoneConverterAut TimeZoneConverter
        {
            get
            {
                if (timeZoneConverter == null)
                {
                    timeZoneConverter = (FXCore.TimeZoneConverterAut)Program.TradeDesk.TimeZoneConverter;
                }
                return timeZoneConverter;
            }
        }
        private static FXCore.TradingSettingsProviderAut tradingSettings;
        public static int GetTradingDayOffset()
        {
            FXCore.PriceHistoryHelperAut priceHistHelper = PriceHistHelper;
            return priceHistHelper.GetTradingDayOffset();
        }
        public static int GetTradingWeekOffset()
        {
            FXCore.PriceHistoryHelperAut priceHistHelper = PriceHistHelper;
            return priceHistHelper.GetTradingWeekOffset();
        }
        public static void GetTradingProperty(List<HostArg> arglist, ref HostArg res)
        {
            try
            {
                if (arglist.Count == 0)
                {
                    res.setError("Wrong number of arguments for getting trading property");
                    return;
                }
                string strPropertyName = arglist[0].getString();
                string sInstrument = string.Empty;
                string strAccountId = string.Empty;
                bool bRet = GetInstrument(strPropertyName, arglist, ref sInstrument);
                if (bRet == false)
                {
                    res.setError("Wrong number of arguments for getting trading property");
                    return;
                }
                bRet = GetAccount(strPropertyName, arglist, ref strAccountId);
                if (bRet == false)
                {
                    res.setError("Wrong number of arguments for getting trading property");
                    return;
                }
                FXCore.PermissionCheckerAut permChecker = PermissionsChecker;
                FXCore.TradingSettingsProviderAut tradingSettings = GetTradingSettings();
                int iRet;
                switch (strPropertyName)
                {
                    case "canCreateEntry":
                        iRet = permChecker.CanCreateEntryOrder(sInstrument);
                        CheckPermissions(iRet, permChecker, ref res);
                        break;
                    case "canCreateMarketOpen":
                        iRet = permChecker.CanCreateMarketOpenOrder(sInstrument);
                        CheckPermissions(iRet, permChecker, ref res);
                        break;
                    case "canCreateMarketClose":
                        iRet = permChecker.CanCreateMarketCloseOrder(sInstrument);
                        CheckPermissions(iRet, permChecker, ref res);
                        break;
                    case "canCreateStopLimit":
                        iRet = permChecker.CanCreateStopLimitOrder(sInstrument);
                        CheckPermissions(iRet, permChecker, ref res);
                        break;
                    case "canCreateNetStopLimit":
                        iRet = permChecker.CanCreateNetStopLimitOrder(sInstrument);
                        CheckPermissions(iRet, permChecker, ref res);
                        break;
                    case "canUseDynamicTrailingForEntryLimit":
                        iRet = permChecker.CanUseDynamicTrailingForEntryLimit();
                        CheckPermissions(iRet, permChecker, ref res); ;
                        break;
                    case "canUseDynamicTrailingForEntryStop":
                        iRet = permChecker.CanUseDynamicTrailingForEntryStop();
                        CheckPermissions(iRet, permChecker, ref res);
                        break;
                    case "canUseDynamicTrailingForLimit":
                        iRet = permChecker.CanUseDynamicTrailingForLimit();
                        CheckPermissions(iRet, permChecker, ref res);
                        break;
                    case "canUseDynamicTrailingForStop":
                        iRet = permChecker.CanUseDynamicTrailingForStop();
                        CheckPermissions(iRet, permChecker, ref res);
                        break;
                    case "canUseFluctuateTrailingForEntryLimit":
                        iRet = permChecker.CanUseFluctuateTrailingForEntryLimit();
                        CheckPermissions(iRet, permChecker, ref res);
                        break;
                    case "canUseFluctuateTrailingForEntryStop":
                        iRet = permChecker.CanUseFluctuateTrailingForEntryStop();
                        CheckPermissions(iRet, permChecker, ref res); ;
                        break;
                    case "canUseFluctuateTrailingForLimit":
                        iRet = permChecker.CanUseFluctuateTrailingForLimit();
                        CheckPermissions(iRet, permChecker, ref res);
                        break;
                    case "canUseFluctuateTrailingForStop":
                        iRet = permChecker.CanUseFluctuateTrailingForStop();
                        CheckPermissions(iRet, permChecker, ref res);
                        break;
                    case "baseUnitSize":
                        iRet = tradingSettings.GetBaseUnitSize(sInstrument, strAccountId);
                        res.setNumber(iRet);
                        break;
                    case "conditionalDistanceEntryLimit":
                        iRet = tradingSettings.GetCondDistEntryLimit(sInstrument);
                        res.setNumber(iRet);
                        break;
                    case "conditionalDistanceEntryStop":
                        iRet = tradingSettings.GetCondDistEntryStop(sInstrument);
                        res.setNumber(iRet);
                        break;
                    case "conditionalDistanceLimitForEntry":
                        iRet = tradingSettings.GetCondDistLimitForEntryOrder(sInstrument);
                        res.setNumber(iRet);
                        break;
                    case "conditionalDistanceStopForEntry":
                        iRet = tradingSettings.GetCondDistStopForEntryOrder(sInstrument);
                        res.setNumber(iRet);
                        break;
                    case "conditionalDistanceLimitForTrade":
                        iRet = tradingSettings.GetCondDistLimitForTrade(sInstrument);
                        res.setNumber(iRet);
                        break;
                    case "conditionalDistanceStopForTrade":
                        iRet = tradingSettings.GetCondDistStopForTrade(sInstrument);
                        res.setNumber(iRet);
                        break;
                    case "marketStatus":
                        iRet = tradingSettings.GetMarketStatus(sInstrument);
                        CheckPermissions(iRet, permChecker, ref res);
                        break;
                    case "maxQuantity":
                        iRet = tradingSettings.GetMaxQuantity(sInstrument, strAccountId);
                        res.setNumber(iRet);
                        break;
                    case "minQuantity":
                        iRet = tradingSettings.GetMinQuantity(sInstrument, strAccountId);
                        res.setNumber(iRet);
                        break;
                    default: 
                        res.setError("Trading property not implemented: " + strPropertyName);
                        break;
                }
            }
            catch (Exception e)
            {
                Console.WriteLine("Exception when getting trading property {0}: {1}", arglist[0].ToString(), e.ToString());
            }
        }
        public static FXCore.TradingSettingsProviderAut GetTradingSettings()
        {
            if (tradingSettings == null)
            {
                tradingSettings = (FXCore.TradingSettingsProviderAut)Program.TradeDesk.TradingSettingsProvider;
            }
            return tradingSettings;
        }
        private static bool GetInstrument(string strPropertyName, List<HostArg> arglist, ref string sInstrument) // extract instrument from trading property's argument list 
        {
            bool bRet = false;
            switch (strPropertyName) 
            {
                case "canCreateEntry":
                case "canCreateMarketOpen":
                case "canCreateMarketClose":
                case "canCreateStopLimit":
                case "baseUnitSize":
                case "conditionalDistanceEntryLimit":
                case "conditionalDistanceEntryStop":
                case "conditionalDistanceLimitForEntry":
                case "conditionalDistanceStopForEntry":
                case "conditionalDistanceLimitForTrade":
                case "conditionalDistanceStopForTrade":
                case "marketStatus":
                case "maxQuantity":
                case "minQuantity":                     // all these trading properties require instrument in the argument list (instrument should be the second argument)
                    if (arglist.Count > 1)
                    {
                        sInstrument = arglist[1].getString();
                        bRet = true;
                    }
                    break;
                default: // do not need instrument for this trading property
                    bRet = true;
                    break;
            }
            return bRet;
        }
        private static bool GetAccount(string strPropertyName, List<HostArg> arglist, ref string strAccount)       // extract account from trading property's argument list
        {
            bool bRet = false;
            switch (strPropertyName)        // get account
            {
                case "canCreateEntry":
                case "canCreateMarketOpen":
                case "canCreateMarketClose":
                case "canCreateStopLimit":
                case "baseUnitSize":
                case "maxQuantity":
                case "minQuantity":         // all these trading properties require account in the argument list (account should be the third argument)
                    if (arglist.Count >  2)
                    {
                        strAccount = arglist[2].getString();
                        bRet = true;
                    }
                    break;
                default:        // do not need account for this trading property
                    bRet = true;
                    break;
            }
            return bRet;
        }
        private static void CheckPermissions(int permission, FXCore.PermissionCheckerAut permChecker, ref HostArg ret)
        {
            try
            {
                if (permission == permChecker.PERMISSION_ENABLED)
                    ret.setBool(true);
                else if (permission == permChecker.PERMISSION_DISABLED)
                    ret.setBool(false);
                else if (permission == permChecker.PERMISSION_HIDDEN)
                    ret.setBool(false);
                else
                    ret.setError("Unknown Permissions Code");
            }
            catch (Exception e)
            {
                Console.WriteLine("Exception while checking permission: {0}", e.Message);
                ret.setError("Exception while checking permission");
            }
        }
        public static string GetPrimaryKey(string strTable) // get primary key for Order2Go table (like "OfferID" for "Offers")
        {
            strTable = strTable.ToLower();
            string strPrimaryKey = string.Empty;
            switch (strTable)
            {
                case "offers":
                case "summary":
                    strPrimaryKey = "OfferID";
                    break;
                case "accounts":
                    strPrimaryKey = "AccountID";
                    break;
                case "orders":
                    strPrimaryKey = "OrderID";
                    break;
                case "trades":
                case "closed trades":
                    strPrimaryKey = "TradeID";
                    break;
                case "messages":
                    strPrimaryKey = "MsgID";
                    break;
                default:
                    break;
            }
            return strPrimaryKey;
        }
    }

}
