using System;
using System.Collections.Generic;
using System.Text;
using gehtsoft.indicore;
using Microsoft.Win32;
using System.Runtime.InteropServices;

namespace ApplyStrategy
{
    class Program
    {
        static IndicatorManager mgr;
        // TODO: *** Change connection and url to those you are using ***
        static string connection = "demo";
        static string url = "www.fxcorporate.com";
        static FXCore.CoreAut core;
        static FXCore.TradeDeskAut tradeDesk;
        public static FXCore.TradeDeskAut TradeDesk
        {
            get { return tradeDesk; }
        }
        static FXCore.TradeDeskEventsSinkClass sink; 
        static int iSubID = -1;
        public static int SubscriptionID
        {
            get { return iSubID; }
        }
        static bool bStopped = false;
        public static bool Stopped
        {
            get { return bStopped; }
            set {bStopped = value;}
        }
        static StrategyCallback callBack;
        static StrategyInstance instance;
        public static StrategyInstance Instance
        {
            get { return Instance; }
        }
        static MyHost host;
        public static MyHost Host
        {
            get {return host;}
        }
        static string sInstrument = "EUR/USD";
        static string sTimeFrame = "m1";

        [DllImport("Kernel32")]
        private static extern bool SetConsoleCtrlHandler(EventHandler handler, bool add); 
        private delegate bool EventHandler(CtrlType sig);
        static EventHandler _handler;
        enum CtrlType
        {
            CTRL_C_EVENT = 0,
            CTRL_BREAK_EVENT = 1,
            CTRL_CLOSE_EVENT = 2,
            CTRL_LOGOFF_EVENT = 5,
            CTRL_SHUTDOWN_EVENT = 6
        }
        private static bool Handler(CtrlType sig)
        {
            switch (sig)
            {
                case CtrlType.CTRL_C_EVENT:
                case CtrlType.CTRL_BREAK_EVENT:
                case CtrlType.CTRL_LOGOFF_EVENT:
                case CtrlType.CTRL_SHUTDOWN_EVENT:
                case CtrlType.CTRL_CLOSE_EVENT:
                    return true;
                default:
                    return false;
            }
        }

        static void Main(string[] args)
        {
            _handler += new EventHandler(Handler);
            SetConsoleCtrlHandler(_handler, true);  // to avoid exception when user closes console

            if (args.Length < 2)
            {
                Console.WriteLine("Not Enough Parameters!");
                Console.WriteLine("USAGE: [application].exe [username] [password] [instrument (optional)] [time frame (optional)]");
                Console.WriteLine("\nPress any key to close the program");
                Console.ReadKey();
                return;
            }
            string sUserName = args[0];
            string sPassword = args[1];
            if (args.Length > 2)
                sInstrument = args[2];
            
            if (args.Length > 3)
                sTimeFrame = args[3];

            try
            {
                mgr = new IndicatorManager();
                core = new FXCore.CoreAut();
                tradeDesk = (FXCore.TradeDeskAut)core.CreateTradeDesk("trader");
                host = new MyHost();
                mgr.setHost(host);

                sink = new FXCore.TradeDeskEventsSinkClass();
                sink.ITradeDeskEvents_Event_OnPriceHistoryCompleted += new FXCore.ITradeDeskEvents_OnPriceHistoryCompletedEventHandler(FxCore_OnPriceHistoryCompleted);
                sink.ITradeDeskEvents_Event_OnPriceHistoryFailed += new FXCore.ITradeDeskEvents_OnPriceHistoryFailedEventHandler(FxCore_OnPriceHistoryFailed);
                sink.ITradeDeskEvents_Event_OnRequestCompleted += new FXCore.ITradeDeskEvents_OnRequestCompletedEventHandler(FXCore_OnRequestCompleted);
                sink.ITradeDeskEvents_Event_OnRequestFailed += new FXCore.ITradeDeskEvents_OnRequestFailedEventHandler(FXCore_OnRequestFailed);
                iSubID = tradeDesk.Subscribe(sink);

                Console.WriteLine("Logging in...");
                tradeDesk.Login(sUserName, sPassword, url, connection); 
                Console.WriteLine("Logged in");

                string path = (string)Registry.GetValue("HKEY_LOCAL_MACHINE\\Software\\CandleWorks\\FXTS2", "InstallPath", string.Empty);
                LoadErrorList errList;
                errList = mgr.loadStrategies(path + "\\strategies\\standard");
                foreach (string sError in errList)
                {
                    Console.WriteLine("Error loading strategies {0}", sError);
                }
                errList = mgr.loadIndicators(path + "\\indicators\\standard");
                foreach (string sError in errList)
                {
                    Console.WriteLine("Error loading indicators {0}", sError);
                }
                StrategiesCollection strategies = mgr.getStrategies();
                // TODO: *** Set the strategy you are using ***
                Strategy strategy = strategies["MA_ADVISOR"];
                IndicatorParametersCollection parameters = strategy.getParameters();
                // TODO: *** Set the parameters of your strategy unless you are taking defaults ***
                parameters["FMA_N"].Value.setInteger(7);
                parameters["SMA_N"].Value.setInteger(25);
                // TODO: *** Uncomment next line if you want strategy to trade ***
                //parameters["CANTRADE"].Value.setString("Yes"); 
                string sAccountID = GetAccountID();
                parameters["ACCOUNT"].Value.setString(sAccountID);
                // set other strategy parameters here

                callBack = new MyStrategyCallBack();

                //prepare ask and bid tick streams for the strategy
                string error;
                tradeDesk.EnablePendingEvents(tradeDesk.EventPriceSessionStatusChange + tradeDesk.EventChange);

                IndicatorSource srcBid;
                IndicatorSource srcAsk;
                HistoryMgr.GetPriceHistory(IntPtr.Zero, sInstrument, "t1", DateTime.FromOADate(0), DateTime.FromOADate(0), 300, true, true, 0, out srcBid, out srcAsk); // pre-load 300 ticks
                MyIndicatorTickSource tickBid = (MyIndicatorTickSource)srcBid;
                MyIndicatorTickSource tickAsk = (MyIndicatorTickSource)srcAsk;

                instance = strategy.getInstance(tickBid, tickAsk, parameters, callBack, out error);  // create strategy instance
                if (error != null)
                {
                    if (error.Length > 0)
                    {
                        Console.WriteLine("Error on creating strategy instance: {0}", error);
                        Console.ReadKey();
                        return;
                    }
                }
                HistoryMgr.Attach2Instance(instance.InternalID, tickBid, true, string.Empty, 0, true);
                HistoryMgr.Attach2Instance(instance.InternalID, tickAsk, true, string.Empty, 0, true);

                while (true)
                {
                    bool bNewPrices;
                    HistoryMgr.ProcessPendingEvents(instance.InternalID, sInstrument, out bNewPrices);

                    if (bNewPrices)
                    {
                        string sErr;
                        instance.update(out sErr); // update strategy on new prices

                        if (Stopped)
                        {
                            break;
                        }

                        if (!string.IsNullOrEmpty(sErr))
                        {
                            Console.WriteLine("Cannot update strategy: {0}", sErr);
                        }
                    }
                }
            }
            catch (Exception e)
            {
                Console.WriteLine("Exception occured: {0}", e.ToString());
            }
            finally
            {
                if (tradeDesk != null)
                {
                    if (iSubID != -1)
                        tradeDesk.Unsubscribe(iSubID);

                    tradeDesk.Logout();
                }
                if (instance != null)
                    instance.Dispose();
                Console.WriteLine("Press any key to close the program");
                Console.ReadKey();
            }
        }

        static void FxCore_OnPriceHistoryCompleted(string request, object rates) // this function would be called on "Price History Completed" event for asynchronous methods 
        {
            FXCore.MarketRateEnumAut gotRates = (FXCore.MarketRateEnumAut)rates;
            List<Ticket> tkts = null;
            if (HistoryMgr.dictHistoryStreams.TryGetValue(instance.InternalID, out tkts))
            {
                foreach (Ticket tkt in tkts)
                {
                    if ((!tkt.HistCompleted) && (tkt.HistRequest == request)) // this is the stream we have to fill with prices we got
                    {
                        DateTime dtLastCandleEnd = DateTime.MinValue;
                        if (tkt.Src.isBar())
                        {
                            MyIndicatorBarSource barSrc = (MyIndicatorBarSource)tkt.Src;
                            object candleStart;
                            object candleEnd;

                            foreach (FXCore.MarketRateAut rate in gotRates)
                            {
                                if (tkt.Src.isBid())
                                {
                                    barSrc.AddLast(rate.StartDate, rate.BidOpen, rate.BidHigh, rate.BidLow, rate.BidClose);
                                }
                                else
                                {
                                    barSrc.AddLast(rate.StartDate, rate.AskOpen, rate.AskHigh, rate.AskLow, rate.AskClose);
                                }
                            }


                            // get the end of the last candle
                            if (tkt.Src.size() > 0)
                            {
                                DateTime dtLast = barSrc.date(barSrc.size() - 1);
                                FXCore.PriceHistoryHelperAut priceHistHelper = Order2Go.PriceHistHelper;
                                priceHistHelper.GetCandle(dtLast, barSrc.getBarSize(), Order2Go.GetTradingDayOffset(), Order2Go.GetTradingWeekOffset(), out candleStart, out candleEnd);
                                dtLastCandleEnd = Convert.ToDateTime(candleEnd);
                            }
                        }
                        else
                        {
                            MyIndicatorTickSource tickSrc = (MyIndicatorTickSource)tkt.Src;
                            foreach (FXCore.MarketRateAut rate in gotRates)
                            {
                                if (tkt.Src.isBid())
                                {
                                    tickSrc.AddLast(rate.StartDate, rate.BidClose);
                                }
                                else
                                {
                                    tickSrc.AddLast(rate.StartDate, rate.AskClose);
                                }
                            }
                            if (tkt.Src.size() > 0)
                            {
                                dtLastCandleEnd = tickSrc.date(tickSrc.size() - 1);
                            }
                        }
                       
                        // add ticks from the buffer
                        for (int i = 0; i < tkt.TickBuffer.size(); i++)
                        {
                            double dblVal = tkt.TickBuffer.getTick(i);
                            DateTime dt = tkt.TickBuffer.date(i);
                            if (dt > dtLastCandleEnd)                           // throw out ticks before end of the last candle
                                HistoryMgr.AddTick2Stream(dt, dblVal, tkt.Src);
                        }

                        tkt.HistCompleted = true;
                        string sNotifyError;
                        instance.notifyFinished(tkt.Cookie, true, string.Empty, out sNotifyError);
                        if (!string.IsNullOrEmpty(sNotifyError))
                            Console.WriteLine("Error on notifyFinished: {0}", sNotifyError);
                    }
                }
            }
        }
        static void FxCore_OnPriceHistoryFailed(string sRequestID, string sError) // this function would be called on "Price History Failed" event for asynchronous methods
        {
            List<Ticket> tkts = null;
            if (HistoryMgr.dictHistoryStreams.TryGetValue(instance.InternalID, out tkts))
            {
                foreach (Ticket tkt in tkts)
                {
                    if ((!tkt.HistCompleted) && (tkt.HistRequest == sRequestID)) // found the ticket
                    {
                        string sNotifyError;
                        instance.notifyFinished(tkt.Cookie, false, sError, out sNotifyError);
                        if (!string.IsNullOrEmpty(sNotifyError))
                        Console.WriteLine("Error on notifyFinished: {0}", sNotifyError);
                    }
                }
            }
        }
        static void FXCore_OnRequestCompleted(string sRequestID)                 // this function would be called on "Request Completed" event for asynchronous methods for trading orders
        {
            FxCore_OnRequestCompletedOrFailed(sRequestID, string.Empty);
        }
        static void FXCore_OnRequestFailed(string sRequestID, string sError)     // this function would be called on "Request Failed" event for asynchronous methods for trading orders
        {
            FxCore_OnRequestCompletedOrFailed(sRequestID, sError);
        }
        static void FxCore_OnRequestCompletedOrFailed(string sRequestID, string sOrderError) // notify strategy about success or failure of asynchronous methods for trading orders
        {
            bool bSuccess = true;
            if (sOrderError != string.Empty)
                bSuccess = false;

            List<OrderInfo> lstOrders = null;
            if (Order2Go.OrdersInfo.TryGetValue(instance.InternalID, out lstOrders))
            {
                for (int i = 0; i < lstOrders.Count; i++)
                {
                    OrderInfo order = lstOrders[i];
                    if (order.OrderRequest == sRequestID)
                    {
                        string sNotifyError;
                        instance.notifyFinished(order.OrderCookie, bSuccess, sOrderError, out sNotifyError);
                        if (!string.IsNullOrEmpty(sNotifyError)) 
                            Console.WriteLine("Error on \"notifyFinished\": {0}", sNotifyError);
                    }
                    if (order.OrderDateTime + TimeSpan.FromMinutes(1) < DateTime.Now)
                        lstOrders.RemoveAt(i); // remove orders older than one minute from "Orders" dictionary
                }
                Order2Go.OrdersInfo[instance.InternalID] = lstOrders; 
            }
        }
        static string GetAccountID() // get the first accountID from "Accounts" table
        {
            string sAccountID = string.Empty;
            MyTradingTable accTable = (MyTradingTable)host.findTable("accounts");
            if (accTable != null)
            {
                TradingDataTableEnumerator rowEnum = accTable.getEnumerator();
                if (rowEnum != null)
                {
                    MyTradingRow accRow = (MyTradingRow)rowEnum.next();
                    if (accRow != null)
                    {
                        HostArg hostArgAccID = accRow.cellValue("AccountID");
                        if (hostArgAccID != null)
                        {
                            sAccountID = hostArgAccID.getString();
                        }
                    }
                }
            }
            return sAccountID;
        }
    }
}
